/*
 * Author: Hiroyuki Chishiro
 * License: 2-Clause BSD
 */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define PORT_NUM 65432
#define HOSTNAME "127.0.0.1"
#define SEND_DATA "Hello World"

int main(void)
{
  struct sockaddr_in server_addr;
  int sock;
  int n;

  memset(&server_addr, 0, sizeof(struct sockaddr_in));

  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("socket");
    return 1;
  }

  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(PORT_NUM);
  server_addr.sin_addr.s_addr = inet_addr(HOSTNAME);

  if (connect(sock, (struct sockaddr *) &server_addr, sizeof(server_addr)) < 0) {
    perror("connect");
    goto end_socket;
  }

  if ((n = send(sock, SEND_DATA, sizeof(SEND_DATA), 0)) < 0) {
    perror("send");
    goto end_socket;
  }

  if (n == sizeof(SEND_DATA)) {
    printf("Sending data is successful.\n");
  } else {
    printf("Sending data is not successful.\n");
  }

end_socket:

  if (close(sock) < 0) {
    perror("close");
    return 2;
  }

  return 0;
}
