/*
 * Author: Hiroyuki Chishiro
 * License: 2-Clause BSD
 */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#define BUFSIZE 1024
#define PORT_NUM 65432
#define BACKLOG 1024

int main(void)
{
  int server_sock;
  struct sockaddr_in server_sockaddr;
  struct sockaddr_in client_sockaddr;
  char buf[BUFSIZE];
  socklen_t len;
  int client_sock;
  int n;

  memset(&server_sockaddr, 0, sizeof(struct sockaddr_in));
  memset(&client_sockaddr, 0, sizeof(struct sockaddr_in));
  memset(buf, 0, sizeof(buf));

  if ((server_sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("socket");
    return 1;
  }

  server_sockaddr.sin_family = AF_INET;
  server_sockaddr.sin_port = htons(PORT_NUM);
  server_sockaddr.sin_addr.s_addr = INADDR_ANY;

  if ((bind(server_sock, (struct sockaddr *) &server_sockaddr,
            sizeof(server_sockaddr))) < 0) {
    perror("bind");
    goto end_server_socket;
  }

  if (listen(server_sock, BACKLOG) < 0) {
    perror("listen");
    goto end_server_socket;
  }

  len = sizeof(client_sockaddr);

  if ((client_sock = accept(server_sock, (struct sockaddr *) &client_sockaddr,
                            &len)) < 0) {
    perror("accept");
    goto end_server_socket;
  }

  if ((n = recv(client_sock, buf, sizeof(buf), 0)) < 0) {
    perror("recv");
    goto end_client_socket;
  }

  printf("%s\n", buf);

end_client_socket:

  if (close(client_sock) < 0) {
    perror("close");

    if (close(server_sock) < 0) {
      return 2;
    } else {
      return 3;
    }
  }

end_server_socket:

  if (close(server_sock) < 0) {
    perror("close");
    return 4;
  }

  return 0;
}
